<?php
if (! defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly
include_once __DIR__."/../lib/Client.php";

use Podeli\Payment\Client;

class WC_Gateway_Podelipayment extends WC_Payment_Gateway
{
    public function __construct()
    {
        $this->id = 'podelipayment';
        $this->has_fields = false;
        $this->order_button_text = 'Оплата через Подели';
        $this->method_title = 'Оплата через Подели';
        $this->method_description = 'Оплата через сервис Подели';
        $this->supports = [
            'products',
            'refunds',
        ];

        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->instructions = $this->get_option('instructions', $this->description);
        $this->test = $this->get_option('test') === 'yes';
        $this->login = $this->get_option('login');
        $this->password = $this->get_option('password');
        $this->cert_path = $this->get_option('cert_path');
        $this->cert_text = $this->get_option('cert_text');
        $this->key_path = $this->get_option('key_path');
        $this->key_text = $this->get_option('key_text');
        $this->enable_log = $this->get_option('log');
        $this->prefix = $this->get_option('prefix');
        $this->use_curl = $this->get_option('user_curl');

        // Actions
        add_action('woocommerce_update_options_payment_gateways_'.$this->id, [$this, 'process_admin_options']);

        // Payment listener/API hook
        add_action('woocommerce_api_wc_gateway_podelipayment', [$this, 'notification']);

        if (! $this->is_valid_for_use()) {
            $this->enabled = false;
        }
    }

    public function is_valid_for_use()
    {
        if (! in_array(get_woocommerce_currency(), apply_filters('woocommerce_podelipayment_supported_currencies', ['RUB']))) {
            return false;
        }

        return true;
    }

    public function admin_options()
    {
        if ($this->is_valid_for_use()) {
            parent::admin_options();
        } else {
            ?>
            <div class="inline error"><p>
                    <strong><?php _e('Gateway Disabled', 'woocommerce'); ?></strong>: <?php _e('Podeli does not support your store currency.', 'podeli_payment'); ?>
                </p></div>
            <?php
        }
    }

    public function init_form_fields()
    {
        $this->form_fields = [
            'enabled' => [
                'title' => __('Enable/Disable', 'woocommerce'),
                'type' => 'checkbox',
                'label' => 'Включить оплату Подели',
                'default' => 'yes',
            ],
            'title' => [
                'title' => __('Title', 'woocommerce'),
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                'default' => 'Оплатить подели',
                'desc_tip' => true,
            ],
            'description' => [
                'title' => __('Description', 'woocommerce'),
                'type' => 'textarea',
                'description' => __('Payment method description that the customer will see on your checkout.', 'woocommerce'),
                'default' => 'Оплачивайте частями через сервис Подели',
                'desc_tip' => true,
            ],
            'log' => [
                'title' => __('Enable/Disable', 'woocommerce'),
                'type' => 'checkbox',
                'label' => 'Включить логи',
                'default' => 'yes',
            ],
            'prefix' => [
                'title' => 'Префикс заявки на оплату',
                'type' => 'text',
                'default' => '',
                'description' => 'Нужен для решения проблемы с дублированием заявок на оплату',
            ],
            'test' => [
                'title' => __('Enable/Disable', 'woocommerce'),
                'type' => 'checkbox',
                'label' => 'Включить тестовый режим',
                'default' => 'yes',
            ],
            'login' => [
                'title' => 'Логин',
                'type' => 'text',
                'default' => '',
            ],
            'password' => [
                'title' => 'Пароль',
                'type' => 'text',
                'default' => '',
            ],
            'cert_path' => [
                'title' => 'Путь к сертификату',
                'type' => 'text',
                'default' => '',
                'description' => 'Путь к файлу сертификата относительно корня сайта',
            ],
            'cert_text' => [
                'title' => 'Сертификат',
                'type' => 'textarea',
                'description' => 'Поле ввода для содержимого файла сертификата (в случае, если нет возможности скопировать файл на сервер и указать путь до него в поле "Путь к сертификату")',
                'default' => '',
            ],
            'key_path' => [
                'title' => 'Путь к ключевому файлу',
                'type' => 'text',
                'default' => '',
                'description' => 'Путь к ключевому файлу относительно корня сайта',
            ],
            'key_text' => [
                'title' => 'Ключ',
                'type' => 'textarea',
                'description' => 'Поле ввода для содержимого файла ключа (в случае, если нет возможности скопировать файл на сервер и указать путь до него в поле "Путь к ключевому файлу")',
                'default' => '',
            ],
            'use_curl' => [
                'title' => 'Использовать curl',
                'type' => 'checkbox',
                'label' => 'Использовать curl для отправки запросов',
                'default' => 'no',
            ],
        ];
    }

    public function process_payment($order_id)
    {
        $order = new WC_Order($order_id);
        $data = $this->prepareData($order);
        $link = $this->createPaymentLink($data);

        return ['result' => 'success', 'redirect' => $link];
    }

    public function process_refund($orderId, $amount = null, $reason = '')
    {
        $order = new WC_Order($orderId);
        $data = $this->prepareRefundData($order);

        $client = $this->initClient();

        $response = $client->refund($this->prefix.$order->get_order_number(), $data);

        $order->update_meta_data('_podeli_refund_id', $response['refund_id']);
        $order->add_order_note(sprintf(__('Refunded %1$s - Refund ID: %2$s', 'woocommerce'), $data['amount'], $response['refund_id']));

        return true;
    }

    private function prepareData($order)
    {
        $orderData = $order->get_base_data();

        $notifyUrl = WC()->api_request_url('WC_Gateway_Podelipayment');
        $notifyUrl = add_query_arg(['order_id' => $order->get_id()], $notifyUrl);
        $successUrl = $this->get_return_url($order);
        $siteUrl = get_site_url();
        
        if (mb_strpos($successUrl, $siteUrl) === false) {
            $successUrl = $siteUrl . '/checkout/' . $successUrl;
        }
        $address[] = $orderData['shipping']['postcode'];
        $address[] = $orderData['shipping']['state'];
        $address[] = $orderData['shipping']['city'];
        $address[] = $orderData['shipping']['address_1'];
        $address[] = $orderData['shipping']['address_2'];
        $address = implode(' ', $address);
        $name[] = $orderData['shipping']['last_name'];
        $name[] = $orderData['shipping']['first_name'];
        $name = implode(' ', $name);
        $data = [
            'order' => [
                'id' => $this->prefix.$order->get_order_number(),
                'amount' => $order->get_total(),
                'prepaidAmount' => 0,
                'items' => $this->getOrderItems($order),
                'isTwoStagePayment' => false,
                'addressDelivery' => $address,
                'recipient' => $name,
                'phoneRecipient' => $this->getPhone($order),
            ],
            'clientInfo' => [
                'firstName' => $order->get_shipping_first_name(),
                'lastName' => $order->get_shipping_last_name(),
                'phone' => $this->getPhone($order),
                'email' => $order->get_billing_email(),
            ],
            'notificationUrl' => $notifyUrl,
            'failUrl' => $order->get_cancel_order_url_raw(),
            'successUrl' => $successUrl,
        ];

        return $data;
    }

    private function prepareRefundData($order)
    {
        $refundItems = [];
        $orderRefunds = $order->get_refunds();
        foreach ($orderRefunds as $refund) {
            if ($refund->get_refunded_payment()) {
                continue;
            }

            foreach ($refund->get_items(['line_item']) as $item_id => $item) {
                $quantity = $item->get_quantity(); // Quantity: zero or negative integer
                if ($quantity >= 0) {
                    continue;
                }
                if ($item->get_subtotal() == 0) {
                    continue;
                }

                $item = [
                    'id' => $item->get_product_id() . $item->get_variation_id(),
                    "refundedQuantity" => $quantity * -1,
                ];
                /*
                if ($this->fiscalisation == 'yes') {
                    $receipt = [];
                    $receipt['payment_method'] = $this->payment_method;
                    $receipt['tax'] = $this->product_vat;
                    if ($this->ffd120) {
                        $receipt['payment_object'] = $this->payment_object;
                        $receipt['measurement_unit'] = __('unit', 'podeli_payment');
                    }
                    $item['receipt'] = $receipt;
                }
                */
                $refundItems[] = $item;
            }

            /*
            foreach ($refund->get_items(['fee']) as $item_id => $item) {
                if ($item->get_total() == 0) {
                    continue;
                }

                $item = [
                    "name" => $item->get_name(),
                    "quantity" => 1,
                    "price" => number_format(round(($item->get_total() + $item->get_total_tax()) * -1, 2), 2, '.', ''),
                ];
                if ($this->fiscalisation == 'yes') {
                    $receipt = [];
                    $receipt['payment_method'] = $this->payment_method;
                    $receipt['tax'] = $this->product_vat;
                    if ($this->ffd120) {
                        $receipt['payment_object'] = $this->payment_object;
                        $receipt['measurement_unit'] = __('unit', 'podeli_payment');
                    }
                    $item['receipt'] = $receipt;
                }
                $refundItems[] = $item;
            }
            */

            foreach ($refund->get_items(['shipping']) as $item_id => $item) {
                if ($item->get_total() == 0) {
                    continue;
                }

                $item = [
                    "id" => 0,
                    "refundedQuantity" => 1,
                ];
                /*
                if ($this->fiscalisation == 'yes') {
                    $receipt = [];
                    $receipt['payment_method'] = $this->payment_method;
                    $receipt['tax'] = $this->delivery_vat;
                    if ($this->ffd120) {
                        $receipt['payment_object'] = $this->delivery_payment_object;
                        $receipt['measurement_unit'] = __('unit', 'podeli_payment');
                    }
                    $item['receipt'] = $receipt;
                }
                */
                $refundItems[] = $item;
            }
        }

        $refundItems = apply_filters('podeli_payment_refund_items', $refundItems);


        /*
        $refundableSum = array_reduce($refundItems, function ($carry, $item) {
            return $carry + $item['quantity'] * $item['price'];
        }, 0);
        */

        $data = [
          'order' => [
            'id' => $order->get_id(),
            'refund' => [
              'id' => Client::generateCorrelationId(),
              'initiator' => 'shop',
              'items' => $refundItems
            ] 
          ]
        ];
        /*
        if ($this->fiscalisation == 'yes') {
            $data['fiscalization_settings'] = ['type' => 'enabled'];
        }
        */
        return $data;
    }

    private function getPhone($order)
    {
        $phone = $order->get_billing_phone();
        $phone = preg_replace("#[^\d]#", "", $phone);
        if (! preg_match("#[7|8]{0,1}(\d{10})#", $phone, $match)) {
            return '';
        }

        return '7'.$match[1];
    }

    private function createPaymentLink($data)
    {
        $client = $this->initClient();
        $response = $client->create($data);

        return $response['redirectUrl'];
    }

    private function getOrderItems($order)
    {
        $positions = [];
        $items = $order->get_items(['line_item', 'fee', 'shipping']);
        foreach ($items as $item) {
            if ($item->get_total() == 0) {
                continue;
            }
            $isDelivery = 0;
            if ($item instanceof WC_Order_Item_Shipping) {
                $isDelivery = 1;
            }

            $position = [
                'id' => $isDelivery == 1 ? 0 : $item->get_product_id() . $item->get_variation_id(),
                'name' => $item->get_name(),
                'quantity' => $item->get_quantity(),
                'amount' => number_format(round(($item->get_total() + $item->get_total_tax()) / $item->get_quantity(), 2), 2, '.', ''),
            ];

            /*
            if ($this->fiscalisation == 'yes') {
                $receipt = [];
                $receipt['payment_method'] = $this->payment_method;
                $receipt['tax'] = ($isDelivery) ? $this->delivery_vat : $this->product_vat;
                if ($this->ffd120) {
                    $receipt['payment_object'] = ($isDelivery) ? $this->delivery_payment_object : $this->payment_object;
                    $receipt['measurement_unit'] = __('unit', 'podeli_payment');
                }
                $position['receipt'] = $receipt;
            }
            */

            $positions[] = $position;
        }

        $positions = apply_filters('podeli_payment_order_items', $positions);

        return $positions;
    }

    public function notification()
    {
        $orderId = $_REQUEST['order_id'];

        if (function_exists("wc_get_order")) {
            $order = wc_get_order($orderId);
        } else {
            $order = new WC_Order($orderId);
        }

        if (! $order) {
            throw new \Exception('Order not found');
        }

        $info = $this->getTransactionInfo($this->prefix.$order->get_order_number());

        $logger = wc_get_logger();
        $context = array( 'source' => 'podeli-payment' );
        $logger->info('notify' . ' = ' . json_encode($info, JSON_UNESCAPED_UNICODE), $context);

        $status = strtolower($info['order']['status']);
        /*
        if ($status === 'wait_for_commit') {
            $this->commitPayment($order);

            exit();
        }

        if ($status !== 'committed') {
            exit();
        }
        */

        if ($order->has_status('completed')) {
            exit();
        }
        if ($status === 'completed' || $status === 'commited') {
            $order->payment_complete(time());
        }
        exit();
    }

    private function getTransactionInfo($orderId)
    {
        $client = $this->initClient();
        $result = $client->info($orderId);

        return $result;
    }

    private function commitPayment($order)
    {
        $data = [
            'amount' => $order->get_total(),
            'items' => $this->getOrderItems($order),
            'prepaid_amount' => 0,
        ];

        $client = $this->initClient();
        $result = $client->commit($this->prefix.$order->get_order_number(), $data);

        return $result;
    }

    private function initClient()
    {
        $api = new Client($this->login, $this->password, $this->test, $this->use_curl);

        if (!empty($this->cert_text)) {
            $fpath = $_SERVER['DOCUMENT_ROOT'] . DIRECTORY_SEPARATOR . 'p.crt';
            if (!file_exists($fpath)) {
                if (is_writable(dirname($fpath))) {
                    file_put_contents($fpath, $this->cert_text);
                } else {
                    throw new \Exception('Can\'t create temporary cert file');
                }
            }
            $api->setCertPath($fpath);
        } else if (!empty($this->cert_path)) {
            $api->setCertPath($this->cert_path);
        } else {
            throw new \Exception('No cert path or string found');
        } 

        if (!empty($this->key_text)) {
            $fpath = $_SERVER['DOCUMENT_ROOT'] . DIRECTORY_SEPARATOR . 'p.key';
            if (!file_exists($fpath)) {
                if (is_writable(dirname($fpath))) {
                    file_put_contents($fpath, $this->key_text);
                } else {
                    throw new \Exception('Can\'t create temporary key file');
                }
            }
            $api->setKeyPath($fpath);     
        } else if (!empty($this->key_path)) {
            $api->setKeyPath($this->key_path);
        } else {
            throw new \Exception('No key path or string found');
        }

        if ($this->enable_log == 'yes') {
            $logger = wc_get_logger();
            $api->setLogger($logger);
        }
        return $api;
    } 
}